import datetime
import hashlib
import json
import shutil
import sys
import tarfile
from pathlib import Path
import boto3
import botocore
from pyspark.context import SparkContext
from awsglue.utils import getResolvedOptions
from awsglue.context import GlueContext
from awsglue.dynamicframe import DynamicFrame
from awsglue.job import Job
from grype_packages import grype_platform


SQLITE_DATABASE_NAME = 'vulnerability.db'
GRYPE_TEMP_STORAGE_PATH = '/tmp/scripts/aws_glue_data'
JOB_STATUS_FILE = '/tmp/scripts/job_status.json'


args = getResolvedOptions(sys.argv, ["JOB_NAME", "s3_path", "glue_database", "bucket_name"])
sc = SparkContext()
glueContext = GlueContext(sc)
logger = glueContext.get_logger()
spark = glueContext.spark_session
job = Job(glueContext)
job.init(args["JOB_NAME"], args)
database = args["glue_database"]
BUCKET_NAME = args["bucket_name"]


def sparkSqlQuery(glueContext, query, mapping, transformation_ctx) -> DynamicFrame:
    """
    Function to create a dynamic frame
    """
    for alias, frame in mapping.items():
        frame.toDF().createOrReplaceTempView(alias)
    result = spark.sql(query)
    return DynamicFrame.fromDF(result, glueContext, transformation_ctx)


def make_dirs(dir_path):
    """
    Function to create the directory
    :param dir_path: file to remove
    """
    dir_path = Path(dir_path)
    dir_path.mkdir(parents=True)


def remove_directory(dir_path):
    """
    Function to remove the directory and its files/child directories
    :param dir_path: directory to remove
    """
    dir_path = Path(dir_path)
    if dir_path.is_dir():
        shutil.rmtree(dir_path)


def get_last_grype_execution_record():
    """
    Function to get the last record from grype execution record by group_id.
    """
    json_content = []
    try:
        s3_client = boto3.client('s3')
        object_str = s3_client.get_object(Bucket=BUCKET_NAME, Key='scripts/job_status.json')
        txt = object_str['Body'].read().decode('utf-8')
        json_content = json.loads(txt)
    except botocore.exceptions.ClientError as e:
        if e.response['Error']['Code'] == "404":
            return []
    return json_content


def create_grype_execution(group_id, version, status_list):
    """
    Function to insert the entry of grype execution into postgres.
    :param group_id: group_id
    :param version: version
    :param status_list: status_list
    :return: returns the execution primary run_id.
    """
    run_id = 1 if not status_list else status_list[-1]['run_id'] + 1
    new_execution = {
        'run_id': run_id,
        'group_id': group_id,
        'version': version,
        'url': '',
        'checksum': '',
        'built': '',
        'status': 'STARTED',
        'created_at': str(datetime.datetime.now()),
        'updated_at': str(datetime.datetime.now())
    }
    status_list.append(new_execution)
    json.dump(status_list, open(JOB_STATUS_FILE, 'w+'), indent=2)
    upload_file_to_s3(JOB_STATUS_FILE, BUCKET_NAME, 'scripts/job_status.json')
    return run_id


def complete_grype_execution(execution_run_id, built, url, checksum, json_data):
    """
    Function to update the status of the grype execution.
    :param execution_run_id: execution primary run_id
    :param built: built date time
    :param url: URL to download the compressed sqlite file.
    :param checksum: checksum of the sqlite data.
    :param json_data: json_data
    """
    for recs in json_data:
        if recs['run_id'] == execution_run_id:
            recs['url'] = url
            recs['checksum'] = checksum
            recs['status'] = 'SUCCESS'
            recs['built'] = built
            recs['updated_at'] = str(datetime.datetime.now())
    json.dump(json_data, open(JOB_STATUS_FILE, 'w+'), indent=2)
    upload_file_to_s3(JOB_STATUS_FILE, BUCKET_NAME, 'scripts/job_status.json')


def generate_checksum_from_bytes(file_pointer):
    """
    Function to generate checksum of the file contents.
    :param file_pointer: file_pointer
    :return checksum: checksum of the file content
    """
    sha256_hash = hashlib.sha256()
    # Read and update hash string value in blocks of 4K
    for byte_block in iter(lambda: file_pointer.read(4096), b""):
        sha256_hash.update(byte_block)
    check_sum = f'sha256:{sha256_hash.hexdigest()}'
    return check_sum


def generate_checksum(source_file):
    """
    Function to generate checksum of the file contents.
    :param source_file: source file path
    :return checksum: checksum of the file content
    """
    with open(source_file, 'rb') as file_pointer:
        check_sum = generate_checksum_from_bytes(file_pointer)
    return check_sum


def save_json(file_name, data):
    """
    Function to save the data into json file
    :param file_name: json file path
    :param data: data
    """
    with open(file_name, 'w', encoding='utf-8') as file_pointer:
        json.dump(data, file_pointer)


def make_tarfile(source_files, target_file):
    """
    Function to create tar.gz file
    :param source_files: list of source files
    :param target_file: file path of the tar file.
    """
    with tarfile.open(target_file, 'w:gz') as tar:
        for source_file in source_files:
            tar.add(source_file, arcname=Path(source_file).name)


def upload_file_to_s3(file_path, bucket_name, object_name=None):
    """Upload a file to S3 bucket

    :param file_path: File to upload
    :param bucket_name: Bucket to upload to
    :param object_name: S3 object name. If not specified then file_name is used
    :return s3_url: URL of s3 file
    """
    # If S3 object_name was not specified, use file_name
    if object_name is None:
        object_name = Path(file_path).name

    # Upload the file
    s3_client = boto3.client('s3')
    s3_client.upload_file(file_path, bucket_name, object_name)
    s3_url = f'https://{bucket_name}.s3.amazonaws.com/{object_name}'
    return s3_url


def get_reference_urls():
    """
    Function to fetch the reference links for all cve's from cve_reference table.
    """
    cve_reference_data = glueContext.create_dynamic_frame.from_catalog(
        database=database,
        table_name='vulndb_public_cve_references',
        transformation_ctx="cve_reference_data",
    )
    url_query = """
                select cve_id,url from vulndb_public_cve_references
                """
    reference_details = sparkSqlQuery(
        glueContext,
        query=url_query,
        mapping={"vulndb_public_cve_references": cve_reference_data},
        transformation_ctx="reference_details",
    )
    ref_df = reference_details.toDF()
    status = ref_df.toPandas()
    cve_status_list = status.values.tolist()
    references = {}
    for items in cve_status_list:
        if items[0] not in references:
            references[items[0]] = []
        references[items[0]].append(items[1])
    return references


def get_cve_detail_data():
    """
    Function to fetch the cve_detail_data from cve_details table.
    """
    cve_detail_data = glueContext.create_dynamic_frame.from_catalog(
        database=database,
        table_name='vulndb_public_cve_details',
        transformation_ctx="cve_detail_data",
    )
    cve_query = """
            select cve_id, description, cvss_severity, cvss_v3_severity, cvss_vector, cvss_score, cvss_exploit_subscore,
            cvss_impact_subscore, cvss_v3_vector, cvss_v3_score, cvss_v3_exploitability_score, cvss_v3_impact_score, created_at
            from vulndb_public_cve_details
            """
    cve_info_list = sparkSqlQuery(
        glueContext,
        query=cve_query,
        mapping={"vulndb_public_cve_details": cve_detail_data},
        transformation_ctx="cve_info_list",
    )
    cve_info_df = cve_info_list.toDF()
    cve_info_status = cve_info_df.toPandas()
    cve_status_list = cve_info_status.values.tolist()

    return cve_status_list


class GrypeWriter:
    """
    GrypeWriter class is used to create paths, checksum, and tar file to store at the given location.
    """

    def __init__(self, version, execution_group_id, job_status):
        """
        Constructor to initialize the variables.
        :param version: version of the grype format.
        :param execution_group_id: execution group_id same for both the versions.
        """
        self.grype_tmp_dir_path = GRYPE_TEMP_STORAGE_PATH
        self.db_file_name = SQLITE_DATABASE_NAME
        self.built_datetime = None
        self.dir_path = None
        self.db_file_path = None
        self.db_metadata_file_path = None
        self.tar_file_path = None
        self.version = version
        self.version_in_int = int(self.version.replace('v', ''))
        self.execution_run_id = None
        self.execution_group_id = execution_group_id
        self.job_status = job_status

    def construct_paths(self):
        """
        method to create paths to store grype formatted data.
        """
        self.execution_run_id = create_grype_execution(self.execution_group_id, self.version_in_int, self.job_status)
        self.dir_path = f'{self.grype_tmp_dir_path}/{self.version}'
        make_dirs(self.dir_path)
        self.db_file_path = f'{self.dir_path}/{self.db_file_name}'
        self.db_metadata_file_path = f'{self.dir_path}/metadata.json'

    def writer(self):
        """
        Method to writer the resultant data into s3 bucket.
        """
        db_checksum = generate_checksum(self.db_file_path)
        db_metadata_info = {
            "built": self.built_datetime,
            "version": self.version_in_int,
            "checksum": db_checksum
        }
        save_json(self.db_metadata_file_path, db_metadata_info)
        self.tar_file_path = f'{self.dir_path}/{self.db_file_name.replace(".", "-")}_{self.version}_{self.built_datetime}.tar.gz'
        make_tarfile([self.db_file_path, self.db_metadata_file_path], self.tar_file_path)
        tar_checksum = generate_checksum(self.tar_file_path)
        object_name = self.tar_file_path.split('/')[-1]
        s3_url = upload_file_to_s3(self.tar_file_path, BUCKET_NAME, object_name)
        complete_grype_execution(self.execution_run_id, self.built_datetime, s3_url, tar_checksum, self.job_status)


class GithubGrypeCollector(grype_platform.GrypeCollector):
    """
    GithubGrypeCollector class
    """
    def __init__(self, sqlite_db_objects, allow_versions, cve_details):
        """
        Constructor of GithubGrypeCollector
        :param sqlite_db_objects: sqlite_db_objects
        :param allow_versions: allow_versions
        """
        super().__init__(sqlite_db_objects, allow_versions)
        self.cve_details = cve_details

    def executor(self):
        github_vulns_data = glueContext.create_dynamic_frame.from_catalog(
            database=database,
            table_name='vulndb_public_package_vulns',
            transformation_ctx="github_vulns_data",
        )
        github_advisory_url__query = """
        select *
            from vulndb_public_package_vulns where source_collector='Github_Collector' and deleted='f' and withdrawn_date is NULL

        """

        github_vulns_records = sparkSqlQuery(
            glueContext,
            query=github_advisory_url__query,
            mapping={"vulndb_public_package_vulns": github_vulns_data},
            transformation_ctx="github_vulns_records",
        )
        github_df = github_vulns_records.toDF()
        advisory_url_list = github_df.toPandas()
        print(f'Total Github Advisory URLs: {str(github_vulns_records.count())}')

        data_list = []
        for _, row in advisory_url_list.iterrows():
            data_list.append(row)
        for version in self.allow_versions:
            lst_vuln, lst_vuln_metadata = grype_platform.GrypeFeed().get_github_data(data_list, version, self.cve_details)
            self.populate_feeds(lst_vuln, lst_vuln_metadata, version=version)


class LinuxGrypeCollector(grype_platform.GrypeCollector):
    """
    LinuxGrypeCollector class
    """

    def __init__(self, sqlite_db_objects, allow_versions, record_source, cve_details_recs):
        """
        Constructor of LinuxGrypeCollector
        :param sqlite_db_objects: sqlite_db_objects
        :param allow_versions: allow_versions
        :param record_source: record_source
        """
        self.cve_details_recs = cve_details_recs
        super().__init__(sqlite_db_objects, allow_versions, record_source)

    def executor(self):
        vendor = grype_platform.LINUX_FLAVORS[self.record_source]['vendor']
        urls_dict = {}
        linux_patch_data = glueContext.create_dynamic_frame.from_catalog(
            database=database,
            table_name='vulndb_public_linux_patch',
            transformation_ctx="linux_patch_data",
        )
        linux_vulns_data = glueContext.create_dynamic_frame.from_catalog(
            database=database,
            table_name='vulndb_public_linux_vulns',
            transformation_ctx="linux_vulns_data",
        )
        linux_os_meta_data = glueContext.create_dynamic_frame.from_catalog(
            database=database,
            table_name='vulndb_public_linux_os_metadata',
            transformation_ctx="linux_os_meta_data",
        )

        if self.record_source in ('ol', 'centos'):
            urls_query = f"""
                SELECT vulndb_public_linux_patch.advisory_url,collect_list(vulndb_public_linux_patch.cve_id)
                FROM vulndb_public_linux_patch JOIN vulndb_public_linux_vulns ON vulndb_public_linux_patch.linux_patch_id
                = vulndb_public_linux_vulns.linux_patch_id
                JOIN vulndb_public_linux_os_metadata ON vulndb_public_linux_vulns.os_id = vulndb_public_linux_os_metadata.os_id
                WHERE vulndb_public_linux_os_metadata.os_vendor = '{vendor}' AND vulndb_public_linux_vulns.deleted='f'
                group by vulndb_public_linux_patch.advisory_url

                """
            url_records = sparkSqlQuery(
                glueContext,
                query=urls_query,
                mapping={"vulndb_public_linux_patch": linux_patch_data,
                         "vulndb_public_linux_vulns": linux_vulns_data,
                         "vulndb_public_linux_os_metadata": linux_os_meta_data},
                transformation_ctx="url_records",
            )
            urls_df = url_records.toDF()
            urls_recs = urls_df.toPandas()
            urls_recs_list = urls_recs.values.tolist()
            for url_lnks in urls_recs_list:
                urls_dict[url_lnks[0]] = set(url_lnks[1])

        linux_query = f"""
            SELECT vulndb_public_linux_patch.cve_id,vulndb_public_linux_patch.package,vulndb_public_linux_patch.fixed_version,
            vulndb_public_linux_patch.advisory_severity,vulndb_public_linux_patch.advisory_url,vulndb_public_linux_patch.advisory_url_alias,
            vulndb_public_linux_patch.description,vulndb_public_linux_os_metadata.os_version,vulndb_public_linux_patch.created_at
            FROM vulndb_public_linux_patch JOIN vulndb_public_linux_vulns ON vulndb_public_linux_patch.linux_patch_id
            = vulndb_public_linux_vulns.linux_patch_id
            JOIN vulndb_public_linux_os_metadata ON vulndb_public_linux_vulns.os_id = vulndb_public_linux_os_metadata.os_id
            WHERE vulndb_public_linux_os_metadata.os_vendor = '{vendor}' AND vulndb_public_linux_vulns.deleted='f'

            """
        linux_records = sparkSqlQuery(
            glueContext,
            query=linux_query,
            mapping={"vulndb_public_linux_patch": linux_patch_data,
                     "vulndb_public_linux_vulns": linux_vulns_data,
                     "vulndb_public_linux_os_metadata": linux_os_meta_data},
            transformation_ctx="linux_records",
        )

        print(f'Total Linux Records: {str(linux_records.count())}')
        linux_data_df = linux_records.toDF()
        linux_recs = linux_data_df.toPandas()
        linux_recs_list = linux_recs.values.tolist()
        for version in self.allow_versions:
            data_dic = {'urls_dict': urls_dict, 'vendor': vendor, 'cve_info': self.cve_details_recs}
            lst_vuln, lst_vuln_metadata = grype_platform.GrypeFeed().get_linux_data(linux_recs_list, version, data_dic, self.record_source)
            self.populate_feeds(lst_vuln, lst_vuln_metadata, version=version)


class NVDGrypeCollector(grype_platform.GrypeCollector):
    """
    NVDGrypeCollector class
    """

    def __init__(self, sqlite_db_objects, allow_versions):
        """
        Constructor of NVD Grype Collector
        :param sqlite_db_objects: sqlite_db_objects
        :param allow_versions: allow_versions
        """
        super().__init__(sqlite_db_objects, allow_versions)

    def executor(self):
        """
        Main method to execute and save the formatted nvd feeds.
        """
        cve_detail_data = glueContext.create_dynamic_frame.from_catalog(
            database=database,
            table_name='vulndb_public_cve_details',
            transformation_ctx="cve_detail_data",
        )
        cve_mapping_data = glueContext.create_dynamic_frame.from_catalog(
            database=database,
            table_name='vulndb_public_cve_affected_cpe_mapping',
            transformation_ctx="cve_mapping_data",
        )
        cpe_product_data = glueContext.create_dynamic_frame.from_catalog(
            database=database,
            table_name='vulndb_public_cpe_products',
            transformation_ctx="cpe_product_data",
        )
        cpe_config_data = glueContext.create_dynamic_frame.from_catalog(
            database=database,
            table_name='vulndb_public_cpe_config',
            transformation_ctx="cpe_config_data",
        )
        cpe_match_data = glueContext.create_dynamic_frame.from_catalog(
            database=database,
            table_name='vulndb_public_cpe_match',
            transformation_ctx="cpe_match_data",
        )
        products_last_created_query = """
        SELECT vulndb_public_cve_affected_cpe_mapping.cve_id, vulndb_public_cve_affected_cpe_mapping.cve_vuln_id,
        vulndb_public_cpe_products.product, max(vulndb_public_cpe_products.created_at),
        vulndb_public_cpe_config.cpe_uri, vulndb_public_cpe_config.version_start_including,
        vulndb_public_cpe_config.version_start_excluding, vulndb_public_cpe_config.version_end_including,
        vulndb_public_cpe_config.version_end_excluding
        FROM vulndb_public_cpe_products JOIN vulndb_public_cpe_match ON vulndb_public_cpe_products.cpe_product_id =
        vulndb_public_cpe_match.cpe_product_id JOIN vulndb_public_cve_affected_cpe_mapping
        ON vulndb_public_cpe_match.cpe_id = vulndb_public_cve_affected_cpe_mapping.product_cpe_id
        JOIN vulndb_public_cpe_config
        ON vulndb_public_cpe_config.cpe_id = vulndb_public_cve_affected_cpe_mapping.product_cpe_id
        WHERE vulndb_public_cve_affected_cpe_mapping.deleted='f'
        GROUP BY vulndb_public_cve_affected_cpe_mapping.cve_id, vulndb_public_cve_affected_cpe_mapping.cve_vuln_id,
        vulndb_public_cpe_products.product, vulndb_public_cpe_config.cpe_uri,
        vulndb_public_cpe_config.version_start_including, vulndb_public_cpe_config.version_start_excluding,
        vulndb_public_cpe_config.version_end_including, vulndb_public_cpe_config.version_end_excluding

        """

        nvd_product_data = sparkSqlQuery(
            glueContext,
            query=products_last_created_query,
            mapping={"vulndb_public_cve_affected_cpe_mapping": cve_mapping_data,
                     "vulndb_public_cve_details": cve_detail_data,
                     "vulndb_public_cpe_products": cpe_product_data,
                     "vulndb_public_cpe_config": cpe_config_data,
                     "vulndb_public_cpe_match": cpe_match_data},
            transformation_ctx="nvd_product_data",
        )
        products_last_created_query = """
        SELECT vulndb_public_cve_affected_cpe_mapping.cve_id,
        vulndb_public_cve_affected_cpe_mapping.cve_vuln_id, vulndb_public_cpe_products.product,
        max(vulndb_public_cpe_products.created_at), vulndb_public_cpe_config.cpe_uri,
        vulndb_public_cpe_config.version_start_including, vulndb_public_cpe_config.version_start_excluding,
        vulndb_public_cpe_config.version_end_including, vulndb_public_cpe_config.version_end_excluding
        FROM vulndb_public_cpe_products JOIN vulndb_public_cpe_match ON vulndb_public_cpe_products.cpe_product_id =
        vulndb_public_cpe_match.cpe_product_id JOIN vulndb_public_cve_affected_cpe_mapping
        ON vulndb_public_cpe_match.cpe_id = vulndb_public_cve_affected_cpe_mapping.os_cpe_id
        JOIN vulndb_public_cpe_config
        ON vulndb_public_cpe_config.cpe_id = vulndb_public_cve_affected_cpe_mapping.os_cpe_id
        WHERE vulndb_public_cve_affected_cpe_mapping.deleted='f'
        GROUP BY vulndb_public_cve_affected_cpe_mapping.cve_id, vulndb_public_cve_affected_cpe_mapping.cve_vuln_id,
        vulndb_public_cpe_products.product, vulndb_public_cpe_config.cpe_uri,
        vulndb_public_cpe_config.version_start_including, vulndb_public_cpe_config.version_start_excluding,
        vulndb_public_cpe_config.version_end_including, vulndb_public_cpe_config.version_end_excluding

        """

        nvd_os_data = sparkSqlQuery(
            glueContext,
            query=products_last_created_query,
            mapping={"vulndb_public_cve_affected_cpe_mapping": cve_mapping_data,
                     "vulndb_public_cve_details": cve_detail_data,
                     "vulndb_public_cpe_products": cpe_product_data,
                     "vulndb_public_cpe_config": cpe_config_data,
                     "vulndb_public_cpe_match": cpe_match_data},
            transformation_ctx="nvd_os_data",
        )

        nvd_recs_prod_merged = nvd_product_data.toDF().join(
            nvd_os_data.select_fields(['cve_id', 'cve_vuln_id', 'cpe_uri']).rename_field('cpe_uri', 'cpe_uri_running_on').toDF(),
            ['cve_id', 'cve_vuln_id'],
            how='left_outer'
        ).toPandas()

        nvd_recs_running_on = nvd_os_data.toDF().toPandas()
        nvd_recs_running_on['cpe_uri_running_on'] = None

        nvd_recs_prod_merged_list = nvd_recs_prod_merged.values.tolist()
        nvd_recs_running_on_list = nvd_recs_running_on.values.tolist()

        total_cpe_products = {}
        cve_products_mapping = {}
        for item in nvd_recs_prod_merged_list:
            cve_id_prod, _, product, created_at = item[0:4]
            if product == 'docker' and cve_id_prod == 'CVE-2017-7297':
                continue
            if cve_id_prod not in cve_products_mapping:
                cve_products_mapping[cve_id_prod] = {}
            if product not in cve_products_mapping[cve_id_prod]:
                cve_products_mapping[cve_id_prod][product] = created_at
            if cve_id_prod not in total_cpe_products:
                total_cpe_products[cve_id_prod] = []
            total_cpe_products[cve_id_prod].append(item[4:])

        cve_products_run_on_os_mapping = {}
        for item in nvd_recs_running_on_list:
            cve_id_prod, _, product, created_at = item[0:4]
            if product == 'docker':
                continue
            if cve_id_prod not in cve_products_run_on_os_mapping:
                cve_products_run_on_os_mapping[cve_id_prod] = {}
            if product not in cve_products_run_on_os_mapping[cve_id_prod]:
                cve_products_run_on_os_mapping[cve_id_prod][product] = created_at
            if cve_id_prod not in total_cpe_products:
                total_cpe_products[cve_id_prod] = []
            total_cpe_products[cve_id_prod].append(item[4:])

        cve_data_list = get_cve_detail_data()
        reference_links = get_reference_urls()

        for version in self.allow_versions:
            vuln_list, vuln_metadata_list = [], []
            for recs in cve_data_list:
                cve_id = recs[0]
                cve_details = {
                    'cve_id': cve_id, 'description': recs[1], 'cvss_severity': recs[2],
                    'cvss_v3_severity': recs[3], 'cvss_vector': recs[4], 'cvss_score': recs[5],
                    'cvss_exploit_subscore': recs[6], 'cvss_impact_subscore': recs[7], 'cvss_v3_vector': recs[8],
                    'cvss_v3_score': recs[9], 'cvss_v3_exploitability_score': recs[10],
                    'cvss_v3_impact_score': recs[11], 'created_at': recs[12]
                }
                links = reference_links[cve_id] if cve_id in reference_links else []
                additional_data = {'cve_prod_map': cve_products_mapping,
                                   'cve_prod_on_os': cve_products_run_on_os_mapping,
                                   'total_cpe_products': total_cpe_products, 'links': links,
                                   'cve_detail': cve_details}
                lst_vuln, lst_vuln_metadata = grype_platform.GrypeFeed().get_nvd_data(version, cve_id, additional_data)
                vuln_list.extend(lst_vuln)
                vuln_metadata_list.extend(lst_vuln_metadata)
            self.populate_feeds(vuln_list, vuln_metadata_list, version=version)


def generate_grype_feeds(sqlite_db_objects, allow_versions):
    """
    Function to generate and populate all the collector feeds in grype format.
    """
    cve_details_recs, cve_description_dict = {}, {}
    cve_status_list = get_cve_detail_data()
    for recs in cve_status_list:
        if recs[0] not in cve_details_recs:
            cve_details_recs[recs[0]] = recs
        if recs[0] not in cve_description_dict:
            cve_description_dict[recs[0]] = recs[1]

    # Github feeds
    print('Grype Github Collector started...')
    github_grype_collector = GithubGrypeCollector(sqlite_db_objects, allow_versions, cve_description_dict)
    github_grype_collector.executor()
    print('Grype Github Collector Completed...')

    # Linux feeds
    for flvr in sorted(grype_platform.LINUX_FLAVORS):
        print(f'Grype Linux Collector started for {flvr}')
        linux_grype_collector = LinuxGrypeCollector(sqlite_db_objects, allow_versions, flvr, cve_details_recs)
        linux_grype_collector.executor()
        print(f'Grype Linux Collector completed for {flvr}')
    print('Grype Linux Collector Completed...')

    # NVD feeds
    print('Grype NVD Collector started...')
    nvd_grype_collector = NVDGrypeCollector(sqlite_db_objects, allow_versions)
    nvd_grype_collector.executor()
    print('Grype NVD Collector Completed...')


def grype_executor_func():
    """
    ---
    get:
      description: Grype feeds collectors, pack the data into grype format and upload to S3 bucket.
      responses:
        '200':
          description: call successful
    """
    print('Grype Collector started...')
    # allow_versions = ('v3', 'v4', 'v5')
    allow_versions = ('v5', )

    # remove all the directory/files from grype tmp storage
    remove_directory(GRYPE_TEMP_STORAGE_PATH)

    # create empty grypte temporary directory again.
    make_dirs(GRYPE_TEMP_STORAGE_PATH)

    status_list = get_last_grype_execution_record()
    execution_group_id = status_list[-1]['group_id'] + 1 if status_list else 1
    sqlite_db_v2, sqlite_db_v3, sqlite_db_v4, sqlite_db_v5 = None, None, None, None

    if 'v2' in allow_versions:
        version = 'v2'
        grype_writer_v2 = GrypeWriter(version, execution_group_id, status_list)
        grype_writer_v2.construct_paths()
        sqlite_db_v2 = grype_platform.SqliteDatabaseV2(grype_writer_v2.db_file_path)
        sqlite_db_v2(grype_writer_v2.built_datetime, grype_writer_v2.version)

    if 'v3' in allow_versions:
        version = 'v3'
        grype_writer_v3 = GrypeWriter(version, execution_group_id, status_list)
        grype_writer_v3.construct_paths()
        sqlite_db_v3 = grype_platform.SqliteDatabaseV3(grype_writer_v3.db_file_path)
        sqlite_db_v3(grype_writer_v3.built_datetime, grype_writer_v3.version)

    if 'v4' in allow_versions:
        version = 'v4'
        grype_writer_v4 = GrypeWriter(version, execution_group_id, status_list)
        grype_writer_v4.construct_paths()
        sqlite_db_v4 = grype_platform.SqliteDatabaseV4(grype_writer_v4.db_file_path)
        sqlite_db_v4(grype_writer_v4.built_datetime, grype_writer_v4.version)

    if 'v5' in allow_versions:
        version = 'v5'
        grype_writer_v5 = GrypeWriter(version, execution_group_id, status_list)
        grype_writer_v5.construct_paths()
        sqlite_db_v5 = grype_platform.SqliteDatabaseV5(grype_writer_v5.db_file_path)
        sqlite_db_v5(grype_writer_v5.built_datetime, grype_writer_v5.version)

    sqlite_db_objects = {
        'v2': sqlite_db_v2,
        'v3': sqlite_db_v3,
        'v4': sqlite_db_v4,
        'v5': sqlite_db_v5
    }

    # trigger collectors
    generate_grype_feeds(sqlite_db_objects, allow_versions)

    if 'v2' in allow_versions:
        grype_writer_v2.built_datetime = datetime.datetime.now().strftime('%Y-%m-%dT%H:%M:%SZ')
        version_in_int = 2
        sqlite_db_v2.insert_records(table='id', records=[(grype_writer_v2.built_datetime, version_in_int)])
        grype_writer_v2.writer()

    if 'v3' in allow_versions:
        version_in_int = 3
        grype_writer_v3.built_datetime = datetime.datetime.now().strftime('%Y-%m-%dT%H:%M:%SZ')
        sqlite_db_v3.insert_records(table='id', records=[(grype_writer_v3.built_datetime, version_in_int)])
        grype_writer_v3.writer()

    if 'v4' in allow_versions:
        version_in_int = 4
        grype_writer_v4.built_datetime = datetime.datetime.now().strftime('%Y-%m-%dT%H:%M:%SZ')
        sqlite_db_v4.insert_records(table='id', records=[(grype_writer_v4.built_datetime, version_in_int)])
        grype_writer_v4.writer()

    if 'v5' in allow_versions:
        version_in_int = 5
        grype_writer_v5.built_datetime = datetime.datetime.now().strftime('%Y-%m-%dT%H:%M:%SZ')
        sqlite_db_v5.insert_records(table='id', records=[(grype_writer_v5.built_datetime, version_in_int)])
        grype_writer_v5.writer()

    print('Grype Collector completed...')
    return {"message": "Grype feeds collector has been completed."}


if __name__ == '__main__':
    grype_executor_func()
    job.commit()
